package com.jiuqi.grid;

import com.jiuqi.util.StreamException;
import com.jiuqi.util.Stream;
import java.io.Serializable;

public class BitList implements Serializable
{
    private byte[] bits;
    private int count;
    private boolean defaultValue;
    
    public BitList(final boolean dftValue) {
        this.bits = null;
        this.count = 0;
        this.defaultValue = dftValue;
    }
    
    private void setCount(final int c) {
        if (c >= 0) {
            int l = (this.bits == null) ? 0 : this.bits.length;
            if (l * 8 >= c) {
                this.count = c;
                return;
            }
            l = (c - 1) / 8 + 1;
            if (this.bits != null) {
                final byte[] nb = new byte[l];
                System.arraycopy(this.bits, 0, nb, 0, this.bits.length);
                this.bits = nb;
            }
            else {
                this.bits = new byte[l];
            }
            this.count = c;
        }
    }
    
    public void add(final boolean value) {
        this.setCount(this.count() + 1);
        this.set(this.count - 1, value);
    }
    
    public boolean get(final int index) {
        if (index >= 0 && index < this.count) {
            final int b = index / 8;
            final int p = index % 8;
            return (this.bits[b] >> p & 0x1) > 0;
        }
        return false;
    }
    
    public void set(final int index, final boolean value) {
        if (this.count <= index) {
            this.setCount(index + 1);
        }
        final int b = index / 8;
        final int p = index % 8;
        if (value) {
            this.bits[b] |= (byte)(1 << p);
        }
        else {
            this.bits[b] &= (byte)~(1 << p);
        }
    }
    
    public void insert(final int index, final int insCount) {
        if (index >= 0 && index <= this.count) {
            this.setCount(this.count + insCount);
            for (int i = this.count - 1; i >= index + insCount; --i) {
                this.set(i, this.get(i - insCount));
            }
            for (int i = index; i < index + insCount; ++i) {
                this.set(i, this.defaultValue);
            }
        }
    }
    
    public void insert(final int index, final int insCount, final boolean value) {
        if (index >= 0 && index <= this.count) {
            this.setCount(this.count + insCount);
            for (int i = this.count - 1; i >= index + insCount; --i) {
                this.set(i, this.get(i - insCount));
            }
            for (int i = index; i < index + insCount; ++i) {
                this.set(i, value);
            }
        }
    }
    
    public void delete(final int index, int delCount) {
        if (index >= 0 && index < this.count) {
            if (index + delCount > this.count) {
                delCount = this.count - index;
            }
            for (int i = index; i < this.count - delCount; ++i) {
                this.set(i, this.get(i + delCount));
            }
            this.setCount(this.count - delCount);
        }
    }
    
    public int count() {
        return this.count;
    }
    
    public void saveToStream(final Stream stream) throws StreamException {
        if (this.count() > 0) {
            stream.writeBuffer(this.bits, 0, (this.count() - 1) / 8 + 1);
        }
    }
    
    public void loadFromStream(final Stream stream, final int size) throws StreamException {
        if (size > 0) {
            stream.readBuffer(this.bits = new byte[size], 0, size);
            this.count = size * 8;
        }
        else {
            this.setCount(0);
            this.bits = null;
        }
    }
}
